//utf8

/*
 * Copyright 2007 (C) Michal Kalewski
 * mkalewski@cs.put.poznan.pl
 *
 * Use, modification, copying and distribution of this software is subject to
 * the terms and conditions of the MIT License.
 * You should have received a copy of the MIT License along with this
 * software; if not, you can download a copy from http://www.opensource.org.
 *
 * For questions, bug reports, feature requests, support requests etc.
 * send an e-mail to: mkalewski@cs.put.poznan.pl.
 *
 * $Id: sms.cpp,v 1.11 2008-01-09 15:52:33 mkalewski Exp $
 *
 */

#include "sms.h"

void PrintVersion() {
        GSMModem::PrintVersion();
}

void PrintAbout() {
        printf("SMSCOMM: free and open source command line and networking tool\n");
        printf("to send and receive SMS messages through a mobile phone or GSM modem.\n\n");
}

void PrintCopyright() {
        printf("Use, modification, copying and distribution of this software is subject to\n");
        printf("the terms and conditions of the MIT License.\n");
        printf("You should have received a copy of the MIT License along with this\n");
        printf("software; if not, you can download a copy from http://www.opensource.org.\n\n");
}

void PrintUsage() {
        printf("Usage:\n   sms -a | -h | -v\n   sms <pin> [ -q | -d ] [ -p <port name> ] <command>\n");
        printf("Options:\n");
        printf("   -a[bout]                  Prints information about this software.\n");
        printf("   -d[ebug]                  Displays debug information during work.\n");
        printf("   -h[elp]                   Prints this message.\n");
        printf("   -p[ort] <port name>       Specifies the serial port (or usb) name\n");
        printf("                             (default port name: /dev/ttyUSB0).\n");
        printf("   -q[uiet]                  Discards all information during work.\n");
        printf("   -v[ersion]                Prints the version message.\n");
        printf("Commands:\n");
        printf("   -e[rase] <num>            Erases SMS message number <num>.\n");
        printf("   -f[irst]                  Reads first unread SMS message.\n");
        printf("   -n[et] [ <port num> ]     Starts TCP networking on the port number <port num>\n");
        printf("                             (default TCP port number: 1234).\n");
        printf("   -r[ead] <num>             Reads SMS message number <num>.\n");
        printf("   -s[end] <gsm num> <text>  Sends SMS message to <gsm num> number.\n\n");
}

bool GSMConnect() {
        logger = new Logger(bQuiet, bDebug);
        if (strlen(cPortName) > 0)
                gsm = new GSMModem(cPortName, logger);
        else
                gsm = new GSMModem(logger);
        if (gsm != NULL)
                return gsm->EnterPIN(cPin);
        return false;
}

void GSMClose() {
        if (gsm != NULL)
                delete gsm;
        if (logger != NULL)
                delete logger;
}

int main(int argc, char* argv[]) {
        bzero(&cPin, sizeof(cPin));
        bzero(&cPortName, sizeof(cPortName));
        bzero(&cGSMNumber, sizeof(cGSMNumber));
        bzero(&iNumber, sizeof(iNumber));
        bzero(&cSMSText, sizeof(cSMSText));
        if (argc < 2 || argc > 8 || (argc == 2 && !((strncmp(argv[1], "-a", 2) == 0) || (strncmp(argv[1], "-h", 2) == 0) ||
                (strncmp(argv[1], "-v", 2) == 0)))) {
                PrintUsage();
                return EXIT_SUCCESS;
        }
        for (int foo = 1; foo < argc; foo++) {
                if (strncmp(argv[foo], "-a", 2) == 0) {
                        PrintVersion();
                        PrintAbout();
                        PrintCopyright();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-h", 2) == 0) {
                        PrintAbout();
                        PrintUsage();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-ver", 4) == 0) {
                        PrintVersion();
                        return EXIT_SUCCESS;
                }
                if (foo == 1) {
                        if (strlen(argv[foo]) <= sizeof(cPin) && SMSNetworking::IsNum(argv[foo])) {
                                strncpy(cPin, argv[foo], strlen(argv[foo]));
                                continue;
                        }
                        PrintUsage();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-q", 2) == 0) {
                        bQuiet = true;
                        continue;
                }
                if (strncmp(argv[foo], "-d", 2) == 0) {
                        bDebug = true;
                        continue;
                }
                if (strncmp(argv[foo], "-p", 2) == 0) {
                        if (argc >= foo+2 && strlen(argv[foo+1]) <= sizeof(cPortName)) {
                                foo++;
                                strncpy(cPortName, argv[foo], strlen(argv[foo]));
                                continue;
                        }
                        PrintUsage();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-e", 2) == 0) {
                        if (argc >= foo+2 && SMSNetworking::IsNum(argv[foo+1])) {
                                foo++;
                                iNumber = atoi(argv[foo]);
                                if (GSMConnect() && gsm->EraseSMS((unsigned short int) iNumber))
                                        printf("%s: sms number %d deleted\n", argv[0], iNumber);
                                else
                                        printf("%s: cannot delete sms number %d!\n", argv[0], iNumber);
                                bWork = true;
                                break;
                        }
                        PrintUsage();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-f", 2) == 0) {
                        SMSRec* sms = new SMSRec();
                        if (GSMConnect())
                                if (gsm->GetFirstUnreadSMS(sms))
                                        sms->PrintSMS();
                                else
                                        printf("%s: no new sms\n", argv[0]);
                        else
                                printf("%s: cannot get sms!\n", argv[0]);
                        if (sms != NULL)
                                delete sms;
                        bWork = true;
                        break;
                }
                if (strncmp(argv[foo], "-r", 2) == 0) {
                        if (argc >= foo+2 && SMSNetworking::IsNum(argv[foo+1])) {
                                foo++;
                                iNumber = atoi(argv[foo]);
                                SMSRec* sms = new SMSRec();
                                if (GSMConnect() && gsm->ReceiveSMS((unsigned short int) iNumber, sms)) {
                                        sms->PrintSMS();
                                }
                                else
                                        printf("%s: cannot read sms number %d!\n", argv[0], iNumber);
                                if (sms != NULL)
                                        delete sms;
                                bWork = true;
                                break;
                        }
                        PrintUsage();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-s", 2) == 0) {
                        if (argc >= foo+3 && strlen(argv[foo+1]) <= sizeof(cGSMNumber) && strlen(argv[foo+2]) <= sizeof(cSMSText)) {
                                foo++; strncpy(cGSMNumber, argv[foo], strlen(argv[foo]));
                                foo++; strncpy(cSMSText, argv[foo], strlen(argv[foo]));
                                if (GSMConnect() && gsm->SendSMS(cGSMNumber, cSMSText))
                                        printf("%s: sms has been sent succesfully\n", argv[0]);
                                else
                                        printf("%s: cannot send sms!\n", argv[0]);
                                bWork = true;
                                break;
                        }
                        PrintUsage();
                        return EXIT_SUCCESS;
                }
                if (strncmp(argv[foo], "-n", 2) == 0) {
                        if (argc >= foo+2)
                                if (SMSNetworking::IsNum(argv[foo+1])) {
                                        foo++; iTcpPortNumber = atoi(argv[foo]);
                                }
                                else
                                        break;
                        bWork = true;
                        if (GSMConnect()) {
                                SMSNetworking* net = new SMSNetworking(iTcpPortNumber, logger, gsm);
                                if (!net->Networking())
                                        return EXIT_FAILURE;
                                if (net != NULL)
                                        delete net;
                        }
                }
        }
        GSMClose();
        if (!bWork)
                PrintUsage();
        return EXIT_SUCCESS;
}

//EOF + NL
