//utf8

/*
 * Copyright 2007 (C) Michal Kalewski
 * mkalewski@cs.put.poznan.pl
 *
 * Use, modification, copying and distribution of this software is subject to
 * the terms and conditions of the MIT License.
 * You should have received a copy of the MIT License along with this
 * software; if not, you can download a copy from http://www.opensource.org.
 *
 * For questions, bug reports, feature requests, support requests etc.
 * send an e-mail to: mkalewski@cs.put.poznan.pl.
 *
 * $Id: smscomm.h,v 1.12 2008-01-11 14:28:59 mkalewski Exp $
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <sys/time.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>
#include "atcommands.h"

#define VERSION "SMSCOMM (1.0 RC; build: 1.0RC/01/2008)"
#define COPYRIGHT "Copyright (C) 2007 Michal Kalewski"

#define SERIALPORTNAME "/dev/ttyUSB0"
#define BAUDRATE B38400

#define SMSFOOTER " SMSCOMM"
#define MAXSMSLENGTH 152
#define DELETEREADSMS false

class Logger
{
        private:
                bool _bQuiet;
                bool _bDebug;
                char _cLastMsg[255];

                void Print(const char*, const char*);

        public:
                Logger(bool, bool);
                void Log(const char*);
                void Info(const char*);
                void Debug(const char*);
                void Warning(const char*);
                void Error(const char*);
                void GetLastMsg(char*, unsigned short int);
};

class SerialPort
{
	private:
		int _iSerialPort;
		char _pcSerialPortName[255];

		bool IsOpen();
		void SerialPortSetings();

        protected:
                Logger* _pLogger;

                bool Open();
                bool Write(const char*);
                bool Read(char*, unsigned short int);

	public:
		SerialPort(Logger*);
		SerialPort(const char*, Logger*);
                ~SerialPort();
};

class SMSRec
{
        public:
                short int _siSMSNumber;
                char _pcSMSStatus[30];
                char _pcSMSGSMNumber[30];
                char _pcSMSSendTime[30];
                char _pcSMSText[512];

                SMSRec();
                bool GetSMS(char*, unsigned short int);
                void PrintSMS();
};

class GSMModem : public SerialPort
{
        private:
                bool _bDevice;
                bool _bDevicePIN;

                bool StartupDevice();
                bool IsReady();
                bool IsReadyPIN();
                bool ATErrorsHandler(const char*);
                bool Response(const char*);
                void SetSMSHeader(const char*, SMSRec*);

        public:
                GSMModem(Logger*);
                GSMModem(const char*, Logger*);
                ~GSMModem();
                static void PrintVersion();
                bool EnterPIN(const char*);
                bool SendSMS(const char*, const char*);
                bool EraseSMS(unsigned short int);
                bool ReceiveSMS(unsigned short int, SMSRec*);
                bool GetFirstUnreadSMS(SMSRec*);
};

//EOF + NL
