//utf8

/*
 * Copyright 2007 (C) Michal Kalewski
 * mkalewski@cs.put.poznan.pl
 *
 * Use, modification, copying and distribution of this software is subject to
 * the terms and conditions of the MIT License.
 * You should have received a copy of the MIT License along with this
 * software; if not, you can download a copy from http://www.opensource.org.
 *
 * For questions, bug reports, feature requests, support requests etc.
 * send an e-mail to: mkalewski@cs.put.poznan.pl.
 *
 * $Id: smsnet.cpp,v 1.5 2008-01-10 16:04:33 mkalewski Exp $
 *
 */

#include "smsnet.h"

//SMSNetworking class

SMSNetworking::SMSNetworking(int iTcpPortNumber, Logger* logger, GSMModem* gsmModem) {
        _logger = logger;
        _gsmModem = gsmModem;
        _iSocket = -1;
        _iClientSocket = -1;
        _iBind = -1;
        _iListen = -1;
        _iTcpPortNumber = iTcpPortNumber;
        bzero(&_stAddr, sizeof(struct sockaddr));
        _stAddr.sin_family = AF_INET;
        _stAddr.sin_addr.s_addr = htonl(INADDR_ANY);
        _stAddr.sin_port = htons(iTcpPortNumber);
}

bool SMSNetworking::IsAlpha(const char* cText) {
        for (int foo = 0; foo < strlen(cText); foo++)
                if (isalpha(cText[foo]))
                        continue;
                else
                        return false;
        return true;
}

bool SMSNetworking::IsNum(const char* cText) {
        for (int foo = 0; foo < strlen(cText); foo++)
                if (isdigit(cText[foo]))
                        continue;
                else
                        return false;
        return true;
}

bool SMSNetworking::InitNetworking() {
        int foo = 1;
        char buf[255];

        if (_logger == NULL || _gsmModem == NULL)
                return false;
        _iSocket = socket(AF_INET, SOCK_STREAM, 0);
        if (_iSocket < 0) {
                sprintf(buf, "InitNetworking(): Can't create a socket! \"%s\"", strerror(errno));
                _logger->Error(buf);
                return false;
        }
        _logger->Debug("Net socket created");
        setsockopt(_iSocket, SOL_SOCKET, SO_REUSEADDR, (char*)foo, sizeof(foo));
        _iBind = bind(_iSocket, (struct sockaddr*)&_stAddr, sizeof(struct sockaddr));
        if (_iBind < 0) {
                sprintf(buf, "InitNetworking(): Can't bind a name to a socket! \"%s\"", strerror(errno));
                _logger->Error(buf);
                if (_iSocket >= 0)
                        close(_iSocket);
                return false;
        }
        _logger->Debug("Net server name bound to the socket");
        _iListen = listen(_iSocket, QUEUESIZE);
        if (_iListen < 0) {
                sprintf(buf, "InitNetworking(): Can't set queue size! \"%s\"", strerror(errno));
                _logger->Error(buf);
                if (_iSocket >= 0)
                        close(_iSocket);
                return false;
        }
        sprintf(buf, "Net server is now working on the port number: %d", _iTcpPortNumber);
        _logger->Log(buf);
        return true;
}

bool SMSNetworking::Networking() {
        int foo = 1;
        int number = -1;
        char smsText[1024];
        char buf[255];
        char logbuf[255];
        char delims[] = "\r";
        char* result = NULL;
        bool quit = false;
        bool run = true;

        if (!InitNetworking())
                return false;
        while (run) {
                _iClientSocket = -1;
                foo = sizeof(struct sockaddr);
                bzero(&_stClientAddr, sizeof(struct sockaddr));
                _iClientSocket = accept(_iSocket, (struct sockaddr*)&_stClientAddr, (socklen_t*)&foo);
                bzero(&buf, sizeof(buf));
                if (_iClientSocket < 0) {
                        sprintf(buf, "Networking(): Can't create a connection's socket! \"%s\"", strerror(errno));
                        _logger->Error(buf);
                        if (_iSocket >= 0)
                                close(_iSocket);
                        return false;
                }
                sprintf(buf, "New connection from: %s", inet_ntoa((struct in_addr)_stClientAddr.sin_addr));
                _logger->Log(buf);
                write(_iClientSocket, cProtocolWELCOME, sizeof(cProtocolWELCOME));
                write(_iClientSocket, cProtocolUSAGE, sizeof(cProtocolUSAGE));
                quit = false;
                while (!quit) {
                        bzero(&buf, sizeof(buf));
                        read(_iClientSocket, buf, sizeof(buf));
                        result = strtok(buf, delims);
                        bzero(&logbuf, sizeof(logbuf));
                        sprintf(logbuf, "New command from %s -- %s",
                                inet_ntoa((struct in_addr)_stClientAddr.sin_addr), result);
                        _logger->Log(logbuf);
                        if (!IsAlpha(result)) {
                                write(_iClientSocket, cProtocolUSAGE, sizeof(cProtocolUSAGE));
                                continue;
                        }
                        if (strncmp(buf, cProtocolERASE, strlen(cProtocolERASE)) == 0) {
                                bzero(&buf, sizeof(buf));
                                read(_iClientSocket, buf, sizeof(buf));
                                result = strtok(buf, delims);
                                if (IsNum(result)) {
                                        number = atoi(buf);
                                        bzero(&buf, sizeof(buf));
                                        if (_gsmModem->EraseSMS((unsigned short int) number)) {
                                                sprintf(buf, "sms number %d deleted\n\n", number);
                                                write(_iClientSocket, buf, strlen(buf)+1);
                                        }
                                        else {
                                                sprintf(buf, "cannot delete sms number %d!\n\n", number);
                                                write(_iClientSocket, buf, strlen(buf)+1);
                                        }
                                }
                                else
                                        write(_iClientSocket, cProtocolUSAGE, sizeof(cProtocolUSAGE));
                                continue;
                        }
                        if (strncmp(buf, cProtocolFIRST, strlen(cProtocolFIRST)) == 0) {
                                SMSRec* sms = new SMSRec();
                                bzero(&smsText, sizeof(smsText));
                                if (_gsmModem->GetFirstUnreadSMS(sms)) {
                                        sms->GetSMS(smsText, sizeof(smsText));
                                        write(_iClientSocket, smsText, strlen(smsText)+1);
                                }
                                else {
                                        bzero(&buf, sizeof(buf));
                                        sprintf(buf, "no new sms\n\n");
                                        write(_iClientSocket, buf, strlen(buf)+1);
                                }
                                if (sms != NULL)
                                        delete sms;
                                continue;
                        }
                        if (strncmp(buf, cProtocolQUIT, strlen(cProtocolQUIT)) == 0) {
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "bye bye!\n\n");
                                write(_iClientSocket, buf, strlen(buf)+1);
                                close(_iClientSocket);
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "Connection from %s closed",
                                        inet_ntoa((struct in_addr)_stClientAddr.sin_addr));
                                _logger->Log(buf);
                                quit = true;
                        }
                        if (strncmp(buf, cProtocolREAD, strlen(cProtocolREAD)) == 0) {
                                bzero(&buf, sizeof(buf));
                                read(_iClientSocket, buf, sizeof(buf));
                                result = strtok(buf, delims);
                                if (IsNum(result)) {
                                        SMSRec* sms = new SMSRec();
                                        number = atoi(buf);
                                        bzero(&smsText, sizeof(smsText));
                                        if (_gsmModem->ReceiveSMS((unsigned short int) number, sms)) {
                                                sms->GetSMS(smsText, sizeof(smsText));
                                                write(_iClientSocket, smsText, strlen(smsText)+1);
                                        }
                                        else {
                                                bzero(&buf, sizeof(buf));
                                                sprintf(buf, "cannot read sms number %d!\n\n", number);
                                                write(_iClientSocket, buf, strlen(buf)+1);
                                        }
                                        if (sms != NULL)
                                                delete sms;
                                }
                                else
                                        write(_iClientSocket, cProtocolUSAGE, sizeof(cProtocolUSAGE));
                                continue;
                        }
                        if (strncmp(buf, cProtocolSEND, strlen(cProtocolSEND)) == 0) {
                                bzero(&buf, sizeof(buf));
                                read(_iClientSocket, buf, sizeof(buf));
                                result = strtok(buf, delims);
                                bzero(&smsText, sizeof(smsText));
                                read(_iClientSocket, smsText, sizeof(smsText));
                                if (strlen(buf) > 1 && strlen(smsText) > 1)
                                        if (_gsmModem->SendSMS(result, smsText)) {
                                                bzero(&buf, sizeof(buf));
                                                sprintf(buf, "sms has been sent succesfully\n\n");
                                                write(_iClientSocket, buf, strlen(buf)+1);
                                        }
                                        else {
                                                bzero(&buf, sizeof(buf));
                                                sprintf(buf, "cannot send sms!\n\n");
                                                write(_iClientSocket, buf, strlen(buf)+1);
                                        }
                                else
                                        write(_iClientSocket, cProtocolUSAGE, sizeof(cProtocolUSAGE));
                                continue;
                        }
                        if (strncmp(buf, cProtocolSHUTDOWN, strlen(cProtocolSHUTDOWN)) == 0) {
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "shutting down SMSCOMM!\n");
                                write(_iClientSocket, buf, strlen(buf)+1);
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "bye bye!\n\n");
                                write(_iClientSocket, buf, strlen(buf)+1);
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "Shutdown received from %s! Shutting down SMSCOMM!",
                                        inet_ntoa((struct in_addr)_stClientAddr.sin_addr));
                                _logger->Warning(buf);
                                close(_iClientSocket);
                                bzero(&buf, sizeof(buf));
                                sprintf(buf, "Connection from %s closed",
                                        inet_ntoa((struct in_addr)_stClientAddr.sin_addr));
                                _logger->Log(buf);
                                quit = true;
                                run = false;
                        }
                        if (!quit)
                                write(_iClientSocket, cProtocolUSAGE, sizeof(cProtocolUSAGE));
                }
        }
        if (_iSocket >= 0)
                close(_iSocket);
        return true;
}

//EOF + NL
